unit StopWtch;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TStopWatch = class(TCustomLabel)
  private
    { Private declarations }
    FEnabled: Boolean;
    FTimeCount: LongInt;
    FHours: Integer;
    FMinutes: Integer;
    FSeconds: Integer;
    // Timer
    FReserved: Byte;
    FInterval: Cardinal;
    FWindowHandle: HWND;
    procedure UpdateTimer;
    procedure SetInterval(Value: Cardinal);
    procedure SetEnabled(Value: Boolean);
    procedure WndProc(var Msg: TMessage);
  protected
    { Protected declarations }
    procedure Timer; dynamic;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Start;
    procedure Stop;
    procedure Reset;
    function GetTime: TDateTime;
  published
    { Published declarations }
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Interval: Cardinal read FInterval write SetInterval default 1000;
    property Align;
    property Alignment;
    property AutoSize;
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    property FocusControl;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowAccelChar;
    property ShowHint;
    property Transparent;
    property Visible;
    property WordWrap;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

uses Consts;

procedure Register;
begin
  RegisterComponents('LivSoft', [TStopWatch]);
end;

constructor TStopWatch.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled := False;
  Width := 70;
  Height := 50;
  Caption := '00:00:00';
  FTimeCount := 0;
  FHours := 0;
  FMinutes := 0;
  FSeconds := 0;
  FInterval := 1000;
  FWindowHandle := AllocateHWnd(WndProc);
end;

destructor TStopWatch.Destroy;
begin
  UpdateTimer;
  DeallocateHWnd(FWindowHandle);
  inherited Destroy;
end;

procedure TStopWatch.WndProc(var Msg: TMessage);
begin
  with Msg do
    if Msg = WM_TIMER then
      try
        Timer;
      except
        Application.HandleException(Self);
      end
    else
      Result := DefWindowProc(FWindowHandle, Msg, wParam, lParam);
end;

procedure TStopWatch.UpdateTimer;
begin
  KillTimer(FWindowHandle, 1);
  if (FInterval <> 0) and FEnabled then
    if SetTimer(FWindowHandle, 1, FInterval, nil) = 0 then
      raise EOutOfResources.CreateRes(SNoTimers);
end;

procedure TStopWatch.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    UpdateTimer;
  end;
end;

procedure TStopWatch.SetInterval(Value: Cardinal);
begin
  if Value <> FInterval then
  begin
    FInterval := Value;
    UpdateTimer;
  end;
end;

procedure TStopWatch.Timer;
var
  szSec, szMin, szHour: String;
begin
  // Update the Caption for the StopWatch
  FSeconds := FSeconds + 1;
  if FSeconds < 10 then
    szSec := '0' + IntToStr(FSeconds)
  else
    szSec := IntToStr(FSeconds);
  if FSeconds = 60 then
  begin
    FSeconds := 0;
    szSec := '00';
    FMinutes := FMinutes + 1;
  end;
  if FMinutes < 10 then
    szMin := '0' + IntToStr(FMinutes)
  else
    szMin := IntToStr(FMinutes);
  if FMinutes = 60 then
  begin
    FMinutes := 0;
    szMin := '00';
    FHours := FHours + 1;
  end;
  if FHours < 10 then
    szHour := '0' + IntToStr(FHours)
  else
    szHour := IntToStr(FHours);
  Caption := szHour + ':' + szMin + ':' + szSec;
end;

procedure TStopWatch.Start;
begin
  FEnabled := True;
  UpdateTimer;
end;

procedure TStopWatch.Stop;
begin
  FEnabled := False;
  UpdateTimer;
end;

procedure TStopWatch.Reset;
begin
  FENabled := False;
  UpdateTimer;
  Caption := '00:00:00';
  FHours := 0;
  FMinutes := 0;
  FSeconds := 0;
end;

function TStopWatch.GetTime;
begin
  Result := StrToTime(Caption);
end;

end.
